/*
 *
 * Xilinx, Inc.
 * XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A 
 * COURTESY TO YOU.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
 * ONE POSSIBLE   IMPLEMENTATION OF THIS FEATURE, APPLICATION OR 
 * STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION 
 * IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE 
 * FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION
 * XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO 
 * THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO 
 * ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE 
 * FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/*****************************************************************************
*
* @Project TFT_Demo
* 
* This project represents an example to use the xps_tft core in order to show some of
* its features. The application loads the bitmap file stored initially in the FLASH 
* memory and displays it in a predefined DDR2 memory area If the bitmap file is not found 
* in the FLASH memory an error message is displayed on both the terminal, LCD screen and
* the screen.
* The application then displays a colorbar (R, G, B) with gradients into another 
* predefined DDR2 memory area.
* Pressing the Up button sets the xps_tft video base address to the colorbar image, 
* so the colorbar is displayed. Pressing the  Down button sets the video base address to 
* the bitmap image therefore the bitmap is displayed on the screen.
*
* @note
* The 256MB DDR2 memory has to be present in order to make the xps_tft core to work
* 
******************************************************************************/


#include <stdio.h>
#include <string.h>
#include "xparameters.h"
#include "xenv_standalone.h"
#include "xbasic_types.h"
#include "xgpio.h"
#include "sleep.h"
#include "xuartns550_l.h"
#include "lcd.h"
#include "xintc.h"
#include "xtft_hw.h"
#include "xtft.h"

/************************** Constant Definitions ****************************/
/**
 * The following constants maps to the XPAR parameter created in the
 * xparameters.h file. They are defined here such that a user can easily
 * change all the needed parameters in one place.
 */

#define TFT_DEVICE_ID	XPAR_TFT_0_DEVICE_ID

//The upper three memory regions of 2MB in the DDR2 are defined to display
//the colorbar, bitmap and a drawing
#define TFT_COLORBAR_ADDR		XPAR_MPMC_0_MPMC_HIGHADDR - 0x001FFFFF
#define TFT_BITMAP_ADDR			XPAR_MPMC_0_MPMC_HIGHADDR - 0x003FFFFF

#define FRAME_SIZE	0x200000

#define DISPLAY_ROW_BUFFER_WIDTH 1024
#define DISPLAY_COLUMNS  640
#define DISPLAY_ROWS     480

/******************************************************************************
 * 
 * 	Variable declaration
 * 
 * ***************************************************************************/

//set to 1 in order to display debug messages
unsigned char verbose=0;

//TFT instance
static XTft TftInstance;

	//LCD globals
	u8 LCD_FirstPrint;
	u8 LCD_LinePos[2];
	u8 * LCD_CurrentLinePos;
	u8 LCD_CurrentLine;

/******************************************************************************
 * 
 * 	Function prototypes
 * 
 * ***************************************************************************/

void LCDInitAndPrint (char * First_Line, char * Second_Line);
int TftWriteString(XTft *InstancePtr, const u8 *CharValue);

int main()
{
   Xuint32 i, j;
   u8 VarChar;
   XTft_Config *TftConfigPtr;
   Xuint32 color_data;
   XStatus Status;
   Xuint32 btn_data;
   Xuint32 VideoBaseAddr;
   u8 BitmapFound;

   /* Initialize RS232_Uart_0 - Set baudrate and number of stop bits */
   XUartNs550_SetBaud(XPAR_RS232_UART_0_BASEADDR, XPAR_XUARTNS550_CLOCK_HZ, 9600);
   XUartNs550_mSetLineControlReg(XPAR_RS232_UART_0_BASEADDR, XUN_LCR_8_DATA_BITS);


   XCACHE_ENABLE_ICACHE();
   XCACHE_ENABLE_DCACHE();
	
   //initialize LCD
   LCDInitAndPrint ((char *) "-  TFT  Demo   -", (char *)"");

   xil_printf ("\r\nTFT Demo");
   
   	/*
	 * Get address of the XTft_Config structure for the given device id.
	 */
	TftConfigPtr = XTft_LookupConfig(TFT_DEVICE_ID);
	if (TftConfigPtr == (XTft_Config *)NULL) {
		if (verbose) xil_printf("\r\nError in TFT initialize");
		return XST_FAILURE;
	}

	/*
	 * Initialize all the TftInstance members and fills the screen with
	 * default background color.
	 */
	Status = XTft_CfgInitialize(&TftInstance, TftConfigPtr,
				 	TftConfigPtr->BaseAddress);
	if (Status != XST_SUCCESS) {
		if (verbose) xil_printf("\r\nError in TFT cfginitialize");
		return XST_FAILURE;
	}

	/*
	 * Wait till Vsync(Video address latch) status bit is set before writing
	 * the frame address into the Address Register. This ensures that the
	 * current frame has been displayed and we can display a new frame of
	 * data. Checking the Vsync state ensures that there is no data flicker
	 * when displaying frames in real time though there is some delay due to
	 * polling.
	 */
	while (XTft_GetVsyncStatus(&TftInstance) !=
					XTFT_IESR_VADDRLATCH_STATUS_MASK);
   
   
   /*
	 * Change the Video Memory Base Address from default value to
	 * the address where the colorbar is displayed
	 */
	XTft_SetFrameBaseAddr(&TftInstance, TFT_COLORBAR_ADDR);
	XTft_ClearScreen(&TftInstance);

 	XTft_SetColor(&TftInstance, 0x00ffffff, 0);
 	
 	//Display a colorbar pattern
 	if (verbose) xil_printf("  Writing Color Bar Pattern\r\n");
 	for (i=0; i <640; i=i+1)
	  {
		  if (i<213)  color_data=(i+42) << 16;
		  else if ((i>213)&&(i<426)) color_data=(i-213+42)<<8;
		  else if (i>426) color_data=(i-426+42);
		  //color_data=0;
		 XTft_FillScreen(&TftInstance,  i, 0, i,479,color_data);
	  }
  
  	//write a text on the screen
    if (verbose) xil_printf ("\r\nWriting Text");
   	XTft_SetPos(&TftInstance, 280, 200);
    Status = TftWriteString(&TftInstance, (u8*)"G E N E S Y S");
	XTft_SetPos(&TftInstance, 240, 220);
    Status = TftWriteString(&TftInstance, (u8*)"C O L O R B A R  D E M O");

	xil_printf("\r\nLoading and decoding bitmap");
	LCDClear();
	LCDPrintString   ("-Loading Bitmap-","");
	Status = Decode_display_bitmap(&TftInstance, TFT_BITMAP_ADDR);

    xil_printf("\r\nConnect a monitor with HDMI input or a DVI monitor through a HDMI - DVI adapter");
    xil_printf("\r\nYou should see the bitmap displayed on the screen, it the bitmap is present in the FLASH memory");


	
	//if the bitmap was not found in the FLASH memory, display a text about that
	if (Status)
	{
		BitmapFound = 0;
		xil_printf("\r\nBitmap was not found in the FLASH memory");
		LCDClear();
		LCDPrintString   ("Bitmap Not Found",
    				      "in FLASH Memory ");
    	XTft_SetFrameBaseAddr(&TftInstance, TFT_BITMAP_ADDR);
    	XTft_ClearScreen(&TftInstance);
 		XTft_SetColor(&TftInstance, 0x00ffffff, 0);
		XTft_SetPos(&TftInstance, 0, 0);
	    Status = TftWriteString(&TftInstance, (u8*)"Bitmap Not Found");
		XTft_SetPos(&TftInstance, 0, 20);
	    Status = TftWriteString(&TftInstance, (u8*)"in FLASH Memory");
	}
	else
	{
		BitmapFound = 1;
		LCDPrintString   ("-  Displaying  -",
    				      "     Bitmap     ");

	}
    xil_printf("\r\nPressing UP button displays a colorbar on the screen");
    xil_printf("\r\nPressing DOWN button displays the bitmap if present");
		
   //poll button data and display the corresponding frame
   while (1)
   {
	 btn_data = XIo_In32(XPAR_PUSH_BUTTONS_7BIT_BASEADDR);
	 
	  if (btn_data)
	    {
		 	//determine which button was pressed
		 	for (i=0; i<5; i++)
		 	{
		 		if (btn_data & (1 << i)) break;	
		 	}
		 	//wait until the button is released
		 	while ((XIo_In32(XPAR_PUSH_BUTTONS_7BIT_BASEADDR) & (1<<i)));
		 	btn_data = 0;
		 	switch (i)
		 	{
		    case 4: //UP button was pressed, change to the colorbar
		    	xil_printf("\r\nDisplaying Colorbar");
		    	VideoBaseAddr = TFT_COLORBAR_ADDR;
		    	XTft_SetFrameBaseAddr(&TftInstance, VideoBaseAddr);
				LCDClear();
				LCDPrintString   ("-  Displaying  -",
    						      "    Colorbar    ");
			break;
			case 2: //DOWN button was pressed, scroll down
		    	VideoBaseAddr = TFT_BITMAP_ADDR;
		    	XTft_SetFrameBaseAddr(&TftInstance, VideoBaseAddr);
				if (BitmapFound)
				{
		    	xil_printf("\r\nDisplaying bitmap");
				LCDPrintString   ("-  Displaying  -",
    						      "     Bitmap     ");
				}
				else
				{
		    	xil_printf("\r\nBitmap was not found in the FLASH memory");
				LCDPrintString   ("Bitmap Not Found",
    						      "in FLASH Memory ");
				}
			break;
		 	}
   		}
   }

   XCACHE_DISABLE_ICACHE();
   XCACHE_DISABLE_DCACHE();
   return 0;
}

/*****************************************************************************
*
* This function initializes the LCD screen and prints a text on it
*
* @param    First_Line, Second_Line are pointers to the strings to be printed
*           on the LCD screen
* 
* @return   None
*
* @note     None
*
******************************************************************************/

void LCDInitAndPrint (char * First_Line, char * Second_Line)
{

   LCDOff();
   LCDClear();
   LCDOn();
   LCDInit();
   LCDPrintString   (First_Line, Second_Line);	
	
}

/*****************************************************************************/
/**
* Write a string of characters to the TFT.
*
* @param	InstancePtr is a pointer to the XTft instance.
* @param	CharValue is a pointer to the character array to be written
*		to the TFT screen.
*
* @return
*		- XST_SUCCESS if successful.
*		- XST_FAILURE if unsuccessful.
*
* @note		None.
*
******************************************************************************/
int TftWriteString(XTft *InstancePtr, const u8 *CharValue)
{
	/*
	 * Writes a character from the string to the screen
	 * until it reaches null or end of the string.
	 */
	while (*CharValue != 0) {
		XTft_Write(InstancePtr, *CharValue);
		CharValue++;
	}

	return XST_SUCCESS;
}
