/*
 *
 * Xilinx, Inc.
 * XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A 
 * COURTESY TO YOU.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
 * ONE POSSIBLE   IMPLEMENTATION OF THIS FEATURE, APPLICATION OR 
 * STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION 
 * IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE 
 * FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION
 * XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO 
 * THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO 
 * ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE 
 * FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY 
 * AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/*****************************************************************************/
/**
*
* @Project USB_EPP_Demo
* 
*
* This project contains an USB-EPP implementation example to test the Genesys
* onboard Digilent USB interface in EPP mode. The application makes use of the
* Adept Register I/O feature. The register adresses are, as follows:
* Write:
*	0x01  LEDs. Data written into this register is displayed on the LEDs
*   0x21 - UART0 (STDOUT). Data written into this register is sent on the terminal.
*  		   Therefore if valid ASCII data is written the corresponding character is
* 		   displayed on the terminal
* 	0x31  LCD. Data written into this register is sent to the LCD screen. Therefore
* 		   if valid ASCII data is written the corresponding character is displayed on
* 		   the LCD screen
* Read:
* 	0x10 - DIP switches. Data read from this register retrieves the status of the DIP 
* 		   switches
*	0x11  Pushbuttons. Data read from this register retrieves the status of the seven 
* 		   pushbuttons (on bits 6..0)
* 
* Writing to any other register address the data is ignored. 
* Reading from any other register returns the last data read or written through the
* USB-EPP interface
* * 
* The USB-EPP interface is implemented in interrupt mode. At the beginning of 
* the program the interrupt controller is initialized and the usb_epp_interrupt_handler()
* interrput handler is registerred.
* 
* Although the usb_epp core implements hardware registers, the interrupt handler implements
* soft registers that are independent from the hardware ones. 
* 
* When a read operation was performed from the USB interface, the interrupt is acknowledged 
* by writing data to the address set in the core address register. 
* 
* When a write operation was performed from the USB interface, the interrupt is acknowledged
* by reading data from the address set in the core address register
*
* @note
*
* Due to the lower speeds of the UART and LCD interfaces file transfer to this 
* peripherals will not work in the application. In order to transfer data from a 
* file to the UART and LCD interfaces a software buffer that stores the data 
* received from the USB-EPP interface is needed
* 
******************************************************************************/


#define USB_EPP_LED_REG	0x01
#define USB_EPP_SWITCH_REG 0x10
#define USB_EPP_BUTTON_REG 0x11
#define USB_EPP_UART_REG 0x21
#define USB_EPP_LCD_REG 0x31


#include <stdio.h>
#include "xparameters.h"
#include "xenv_standalone.h"
#include "xbasic_types.h"
#include "xgpio.h"
#include "xuartns550_l.h"
#include "sleep.h"
#include "lcd.h"
#include "usb_epp.h"
#include "xintc.h"
#include "intc_header.h"



/******************************************************************************
 * 
 * 	Variable declaration
 * 
 * ***************************************************************************/


//interrupt controller
XIntc Intc;

//LCD globals
u8 LCD_FirstPrint;
u8 LCD_LinePos[2];
u8 * LinePos;
u8 LCD_CurrentLine;


/******************************************************************************
 * 
 * 	Function prototypes
 * 
 * ***************************************************************************/

void LCDPrintandShift(char c, u8 * FirstPrint, u8 * LinePos, u8 * CurrentLine);
void LCDInitAndPrint (char * First_Line, char * Second_Line);


/*****************************************************************************
*
* This function represents the interrupt handler for the EPP-USB component
* When an interrupt occurs, first the status is read to determine if a read 
* operation were requested a write operation were performed from the USB-EPP
* interface.
* After that the address is handled and data is written according to the 
* address set
* To acknowledge the interrupt, if a read operation were requested, data has 
* to be written in the data register of the USB-EPP core. The data is automatically 
* written in the hardware register of the USB-EPP core corresponding to the current 
* address. 
* If a write operation were performed, in order to acknowledge the interrupt 
* routine data has to be read from the data register of the USB-EPP core. Data is 
* automatically read from the hardware register of the USB-EPP core corresponding
* to the current address
* 
* @param    None
*
* @return   None
*
* @note     None
*
******************************************************************************/

void usb_epp_interrupt_handler(void * baseaddr_p) {

Xuint32 Usb_epp_data;
Xuint32 Usb_epp_address;
Xuint32 Usb_epp_status;


	Usb_epp_status = XIo_In32(XPAR_USB_EPP_IF_0_BASEADDR + USB_EPP_STATUS_REG_OFFSET);
	//xil_printf("\r\nUSB_EPP_Status = 0x%0.8X", 	Usb_epp_status);
	
	if (Usb_epp_status & USB_EPP_STATUS_READ_REQUESTED)
	{
		//read first the address from where read is requested
		Usb_epp_address = XIo_In32 (XPAR_USB_EPP_IF_0_BASEADDR + USB_EPP_ADDRESS_REG_OFFSET); 
		
		switch (Usb_epp_address)
		{
		case USB_EPP_SWITCH_REG: Usb_epp_data = XIo_In32 (XPAR_DIP_SWITCHES_8BIT_BASEADDR);
		break;
		case USB_EPP_BUTTON_REG: Usb_epp_data = XIo_In32 (XPAR_PUSH_BUTTONS_7BIT_BASEADDR);
		break;
		}
		//acknowledge the interrupt by writing into the specified address
		XIo_Out32(XPAR_USB_EPP_IF_0_BASEADDR + USB_EPP_DATA_REG_OFFSET, Usb_epp_data);
	}
	
	else if (Usb_epp_status & USB_EPP_STATUS_WRITE_PERFORMED)
	{
		//read first the address from where write was perfomed
		Usb_epp_address = XIo_In32 (XPAR_USB_EPP_IF_0_BASEADDR + USB_EPP_ADDRESS_REG_OFFSET); 
		//acknowledge the interrupt by reading from the current address
		Usb_epp_data = XIo_In32(XPAR_USB_EPP_IF_0_BASEADDR + USB_EPP_DATA_REG_OFFSET);
		
		switch (Usb_epp_address)
			{
				case USB_EPP_LED_REG: XIo_Out32(XPAR_LEDS_8BIT_BASEADDR, Usb_epp_data);
				break;
				case USB_EPP_UART_REG: XUartNs550_SendByte(XPAR_RS232_UART_0_BASEADDR, Usb_epp_data);
				break;
				case USB_EPP_LCD_REG:
					LCDPrintandShift(Usb_epp_data, &LCD_FirstPrint, LinePos, &LCD_CurrentLine);
				break;
			}	
	}
	//re-read the status of the USB-EPP interface
	Usb_epp_status = XIo_In32(XPAR_USB_EPP_IF_0_BASEADDR + USB_EPP_STATUS_REG_OFFSET);
    //xil_printf("\r\nUSB_EPP_Status = 0x%0.8X", 	Usb_epp_status);
}

int main()
{
Xuint32 Usb_epp_status;

	XStatus Status;

   /* Initialize RS232_Uart_0 - Set baudrate and number of stop bits */
   XUartNs550_SetBaud(XPAR_RS232_UART_0_BASEADDR, XPAR_XUARTNS550_CLOCK_HZ, 9600);
   XUartNs550_mSetLineControlReg(XPAR_RS232_UART_0_BASEADDR, XUN_LCR_8_DATA_BITS);


   XCACHE_ENABLE_ICACHE();
   XCACHE_ENABLE_DCACHE();

   //initialize LCD
   LCDInitAndPrint ((char *) &"-    GENESYS   -", (char *) &"- USB-EPP Demo -");

   //initialize LCD globals		
   LCD_FirstPrint = 1;
   LCD_LinePos[0] = 0;
   LCD_LinePos[1] = 0;
   LinePos = LCD_LinePos;
   LCD_CurrentLine = 0;

	
	
	//set LEDs as output
	XIo_Out32(XPAR_LEDS_8BIT_BASEADDR + 0x04 , 0x00);
	
	//initialize interrupt controller
	Status = XIntc_Initialize (&Intc, XPAR_XPS_INTC_0_DEVICE_ID);
    if (Status != XST_SUCCESS) xil_printf ("\r\nInterrupt controller initialization failure");
    else xil_printf("\r\nInterrupt controller initialized");
   
    //register usb_epp_interrupt_handler() for the system
    Status = XIntc_Connect (&Intc, XPAR_XPS_INTC_0_USB_EPP_IF_0_EPP_IRPT_INTR,
                             (XInterruptHandler) usb_epp_interrupt_handler,
                             (void *)XPAR_USB_EPP_IF_0_BASEADDR);
    if (Status != XST_SUCCESS) xil_printf ("\r\nRegistering USB_EPP Interupt Failed");
    else xil_printf("\r\nUSB_EPP Interrupt registerred");
  
    //enable interrupt controller
    XIntc_Enable  (&Intc, XPAR_XPS_INTC_0_USB_EPP_IF_0_EPP_IRPT_INTR); 
   
    //start the interrupt controller in real mode
    Status = XIntc_Start(&Intc, XIN_REAL_MODE);
    if (Status != XST_SUCCESS) xil_printf ("\r\nInterupt controller starting failed");
    else xil_printf("\r\nInterrupt controller started");


    //enable interrupts for the processor
    microblaze_enable_interrupts();
  

    xil_printf ("\r\n\r\nUse Adept in Register I/O mode only");
    xil_printf("\r\nWrite from Adept to the following registers");
    xil_printf("\r\n0x01: write to the LEDs");
    xil_printf("\r\n0x21: write to UART0 (Terminal)");
    xil_printf("\r\n0x31: write ASCII data to LCD");
    xil_printf("\r\n\r\nReading from Adept from the following registers:");
    xil_printf("\r\n0x10: Read DIP switch data");
    xil_printf("\r\n0x11: Read Pushbutton data\r\n");
	
	//infinite loop, do nothing and wait for interrupts from the USB-EPP core
    while (1)
    {

    }


   XCACHE_DISABLE_ICACHE();
   XCACHE_DISABLE_DCACHE();
   return 0;
}

/*****************************************************************************
*
* This function prints a character on the LCD screen. At the first character printed
* the LCD screen is cleared and cursor blink is enabled. If the number of characters
* already printed on the LCD screen is larger than 16, the screen is shifted to the 
* left. If the number of characters already printed on a line is larger than 40 
* i.e. the current line is full, the LCD is set on the next line and the cursor is 
* moved to the beginning. 
*
* @param    c 			is the character to be printed
* 			FirstPrint 	is a pointer to a variable that shows whether a character was
*						previously printed or not
* 			LinePos		is a pointer to a u8 array of two, each element indicating the 
* 						current character position in the array
* 			CurrentLine is a poiter to a variable that represents the current LCD line
* 						(0 - first line, 1 - second line)
* 
* @return   None
*
* @note     None
*
******************************************************************************/

void LCDPrintandShift(char c, u8 * FirstPrint, u8 * LinePos, u8 * CurrentLine)
{

u8 * CharPos;

	
	CharPos = LCD_LinePos + (*CurrentLine);

		if (*FirstPrint)
		{
		  *FirstPrint = 0;
		  LCDClear();
		  LCDEnableCursorBlink();
		}
         (*CharPos)++;
    	 if (*CharPos <= 15 )
		 LCDPrintChar(c);
         //shift the LCD display if the number of displayed characters is larger than 16
         else if (*CharPos <= 40)
		 {
	       	LCDPrintChar(c);
            LCDEnableDisplayShift();
		 }
		 else//change the line and disable display shift if the current line is full
		 {
		  	*CharPos=0;
		    * CurrentLine = ! (* CurrentLine);
	    	LCDSetLine((*CurrentLine)+1);
	    	CharPos = LinePos + (*CurrentLine);
	    	*CharPos=1;
	    	LCDDisableDisplayShift();
	    	LCDPrintChar(c);
		 }
}

/*****************************************************************************
*
* This function initializes the LCD screen and prints a text on it
*
* @param    First_Line, Second_Line are pointers to the strings to be printed
*           on the LCD screen
* 
* @return   None
*
* @note     None
*
******************************************************************************/

void LCDInitAndPrint (char * First_Line, char * Second_Line)
{

   LCDOff();
   LCDClear();
   LCDOn();
   LCDInit();
   LCDPrintString   (First_Line, Second_Line);	
	
}




